package meerkat.voting.gui;

import javafx.application.Application;
import javafx.stage.Stage;
import meerkat.voting.gui.configuration.VotingBoothConfiguration;
import meerkat.voting.gui.configuration.VotingBoothConfigurationCreator;
import meerkat.voting.gui.managment.ChainBuilder;
import org.json.JSONException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;


/**
 * Created by Vladimir Eliezer Tokarev on 8/27/2016.
 * The current voting-booth intialization process is next:
 * Main calls to ChainBuilder which initilizes all the TwoWayNode's
 */
public class Main extends Application {

    private final Logger logger = LoggerFactory.getLogger(Main.class);

    /**
     * Creates the VotingBoothConfigurationCreator and creates configuration based on given path
     * @return VotingBoothVonfiguration parsed from json file
     * @throws IOException will be thrown if the configuraiton file doesnt exist
     * @throws JSONException will be thrown if the json is unvalid
     */
    private VotingBoothConfiguration GetConfig() throws IOException, JSONException {
        VotingBoothConfigurationCreator creator = new VotingBoothConfigurationCreator();
        return creator.CreateConfiguration("/F:/vova/meerkat/meerkat-java/voting-booth-gui/src/main/resources/configuration/VotingBoothConfiguration.json");
    }

    @Override
    public void start(Stage primaryStage) throws Exception
    {
        this.logger.debug("Running the app.");
        ChainBuilder.Build(primaryStage, GetConfig());
        primaryStage.setTitle("Meerkat Polling Station");
        primaryStage.show();
    }

    public static void main(String[] args) throws InterruptedException {
        launch(args);
    }

}

