package meerkat.voting.gui.configuration;

import com.google.protobuf.ByteString;
import javafx.embed.swing.SwingFXUtils;
import meerkat.protobuf.BallotQuestionUIElementOuterClass;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;

/**
 * Created by Vladimir Eliezer Tokarev on 10/1/2016.
 * This object contains all the questions and configurations needed for the voting-booth-gui work
 */
public class VotingBoothConfiguration {
    public BallotQuestionUIElementOuterClass.BallotQuestionUIElement NameSelectionQuestion;
    public BallotQuestionUIElementOuterClass.BallotQuestionUIElement NameSelectionByPictureQuestion;

    /**
     * Checks that the configuration that have been given is valud
     * @return true if the configuration was valid else false
     */
    public boolean IsConfigurationValid(){
        // check that both the ballot questions really exists
        if (this.NameSelectionByPictureQuestion == null || this.NameSelectionQuestion == null) {
            return false;
        }
        // check that both the questions answers exists
        if (this.NameSelectionQuestion.getAnswers().getAnswers().getAnswersList() == null ||
                this.NameSelectionByPictureQuestion.getAnswers().getAnswers().getAnswersList() == null) {
            return false;
        }
        // check that both the types of the questions answers are supported
        if (BallotQuestionUIElementOuterClass.ValueType.forNumber(NameSelectionByPictureQuestion.getAnswers().getAnswersType()) == null ||
                BallotQuestionUIElementOuterClass.ValueType.forNumber(NameSelectionQuestion.getAnswers().getAnswersType()) == null ){
            return false;
        }
        // Check that all the given answers are valid bte strings that can be converted into strings
        for (ByteString answer: NameSelectionQuestion.getAnswers().getAnswers().getAnswersList()){
            try {
                String strAnswer = answer.toStringUtf8();
            } catch (Exception e){
                return false;
            }
        }
        // Check that all given byteStrings that are pictures can be converted into pictures
        for (ByteString answer :NameSelectionByPictureQuestion.getAnswers().getAnswers().getAnswersList()){
            try {
                BufferedImage bufferedImage = ImageIO.read(answer.newInput());
                SwingFXUtils.toFXImage(bufferedImage, null);
            } catch (Exception e) {
                return false;
            }
        }

        return true;
    }

}

